@@ -23,6 +23,10 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "cpu.h"
 #include "settings.h"
 #include "mode.h"
@@ -60,14 +64,16 @@ cpugraph_construct (XfcePanelPlugin *plugin)
 
     Ptr<CPUGraph> base = create_gui (plugin);
 
-    read_settings (plugin, base);
+    Settings::init (plugin, base);
+
+    Settings::read (plugin, base);
 
     xfce_panel_plugin_menu_show_about (plugin);
     xfce_panel_plugin_menu_show_configure (plugin);
 
     xfce4::connect_about           (plugin, [base](XfcePanelPlugin *p) { about_cb(); });
     xfce4::connect_free_data       (plugin, [base](XfcePanelPlugin *p) { shutdown(base); });
-    xfce4::connect_save            (plugin, [base](XfcePanelPlugin *p) { write_settings(p, base); });
+    xfce4::connect_save            (plugin, [base](XfcePanelPlugin *p) { Settings::write(p, base); });
     xfce4::connect_configure_plugin(plugin, [base](XfcePanelPlugin *p) { create_options(p, base); });
     xfce4::connect_mode_changed    (plugin, [base](XfcePanelPlugin *p, XfcePanelPluginMode mode) { mode_cb(p, base); });
     xfce4::connect_size_changed    (plugin, [base](XfcePanelPlugin *p, guint size) { return size_cb(p, size, base); });
@@ -129,6 +135,7 @@ create_gui (XfcePanelPlugin *plugin)
     base->has_bars = false;
     base->has_barcolor = false;
     base->bars.orientation = orientation;
+    base->stats_smt = STATS_SMT_BY_DEFAULT;
     base->highlight_smt = HIGHLIGHT_SMT_BY_DEFAULT;
     base->per_core_spacing = PER_CORE_SPACING_DEFAULT;
 
@@ -149,6 +156,7 @@ about_cb ()
         "Agustin Ferrin Pozuelo <gatoguan-os@yahoo.com>",
         "Alexander Nordfelth <alex.nordfelth@telia.com>",
         "Angelo Miguel Arrifano <miknix@gmail.com>",
+        "Błażej Szczygieł <mumei6102@gmail.com>",
         "Florian Rivoal <frivoal@gmail.com>",
         "Jan Ziak <0xe2.0x9a.0x9b@xfce.org>",
         "Ludovic Mercier <lidiriel@coriolys.org>",
@@ -163,7 +171,7 @@ about_cb ()
         "program-name", PACKAGE_NAME,
         "comments", _("Graphical representation of the CPU load"),
         "website", "https://docs.xfce.org/panel-plugins/xfce4-cpugraph-plugin",
-        "copyright", _("Copyright (c) 2003-2021\n"),
+        "copyright", "Copyright \302\251 2004-2023 The Xfce development team",
         "authors", auth, NULL);
 }
 
@@ -199,6 +207,11 @@ CPUGraph::~CPUGraph()
     g_info ("%s", __PRETTY_FUNCTION__);
     for (auto hist_data : history.data)
         g_free (hist_data);
+    if (channel)
+    {
+        g_object_unref (channel);
+        Settings::finalize ();
+    }
 }
 
 static void
@@ -383,7 +396,7 @@ detect_smt_issues (const Ptr<CPUGraph> &base)
             g_info ("actual_load[%u] = %g", i, actual_load[i]);
     }
 
-    if (base->topology && base->topology->smt)
+    if (G_LIKELY (base->topology && base->topology->smt))
     {
         /* Use <Topology> instead of <const Topology>.
          * The non-const version results in less efficient C++ code,
@@ -611,7 +624,8 @@ update_cb (const Ptr<CPUGraph> &base)
     if (!read_cpu_data (base->cpu_data))
         return xfce4::TIMEOUT_AGAIN;
 
-    detect_smt_issues (base);
+    if (base->topology && base->topology->smt && base->isSmtIssuesEnabled ())
+        detect_smt_issues (base);
 
     if (!base->history.data.empty())
     {
@@ -621,10 +635,14 @@ update_cb (const Ptr<CPUGraph> &base)
         base->history.offset = (base->history.offset - 1) & base->history.mask();
         for (guint core = 0; core < base->nr_cores + 1; core++)
         {
-            CpuLoad load;
+            const CpuData &cpu_core_data = base->cpu_data[core];
+            CpuLoad &load = base->history.data[core][base->history.offset];
             load.timestamp = timestamp;
-            load.value = base->cpu_data[core].load;
-            base->history.data[core][base->history.offset] = load;
+            load.value = cpu_core_data.load;
+            load.system = cpu_core_data.system;
+            load.user = cpu_core_data.user;
+            load.nice = cpu_core_data.nice;
+            load.iowait = cpu_core_data.iowait;
         }
     }
 
@@ -637,7 +655,7 @@ update_cb (const Ptr<CPUGraph> &base)
 static void
 update_tooltip (const Ptr<CPUGraph> &base)
 {
-    auto tooltip = xfce4::sprintf (_("Usage: %u%%"), (guint) roundf (base->cpu_data[0].load * 100));
+    auto tooltip = xfce4::sprintf (_("CPU usage: %.1f%%"), base->cpu_data[0].load * 100.0f);
     if (gtk_label_get_text (GTK_LABEL (base->tooltip_text)) != tooltip)
         gtk_label_set_text (GTK_LABEL (base->tooltip_text), tooltip.c_str());
 }
@@ -948,6 +966,8 @@ CPUGraph::set_nonlinear_time (const Ptr<CPUGraph> &base, bool non_linear)
     if (base->non_linear != non_linear)
     {
         base->non_linear = non_linear;
+        if (!non_linear)
+            base->non_linear_cache = {};
         queue_draw (base);
     }
 }
@@ -978,6 +998,12 @@ CPUGraph::set_per_core_spacing (const Ptr<CPUGraph> &base, guint spacing)
     }
 }
 
+void
+CPUGraph::set_stats_smt (const Ptr<CPUGraph> &base, bool stats_smt)
+{
+    base->stats_smt = stats_smt;
+}
+
 void
 CPUGraph::set_smt (const Ptr<CPUGraph> &base, bool highlight_smt)
 {
@@ -1004,6 +1030,13 @@ CPUGraph::set_update_rate (const Ptr<CPUGraph> &base, CPUGraphUpdateRate rate)
     }
 }
 
+void
+CPUGraph::maybe_clear_smt_stats (const Ptr<CPUGraph> &base)
+{
+    if (!base->isSmtIssuesEnabled ())
+        base->stats = {};
+}
+
 void
 CPUGraph::set_size (const Ptr<CPUGraph> &base, guint size)
 {
@@ -1030,6 +1063,8 @@ void
 CPUGraph::set_mode (const Ptr<CPUGraph> &base, CPUGraphMode mode)
 {
     base->mode = mode;
+    base->nearest_cache = {};
+    base->non_linear_cache = {};
     if (mode == MODE_DISABLED)
     {
         gtk_widget_hide (base->frame_widget);
