@@ -29,6 +29,7 @@
 #endif
 
 #include <libxfce4panel/libxfce4panel.h>
+#include <xfconf/xfconf.h>
 #include <string>
 #include <vector>
 #include "xfce4++/util.h"
@@ -39,6 +40,7 @@ using xfce4::Ptr;
 using xfce4::Ptr0;
 
 #define BORDER 8
+#define STATS_SMT_BY_DEFAULT false
 #define HIGHLIGHT_SMT_BY_DEFAULT false
 #define MAX_HISTORY_SIZE (100*1000)
 #define MAX_LOAD_THRESHOLD 0.2
@@ -51,11 +53,19 @@ using xfce4::Ptr0;
 
 enum CPUGraphMode
 {
-    MODE_DISABLED = -1,
-    MODE_NORMAL = 0,
-    MODE_LED = 1,
-    MODE_NO_HISTORY = 2,
-    MODE_GRID = 3,
+    MODE_DISABLED = 0,
+    MODE_NORMAL = 1,
+    MODE_LED = 2,
+    MODE_NO_HISTORY = 3,
+    MODE_GRID = 4,
+};
+
+enum CPUGraphColorMode
+{
+    COLOR_MODE_SOLID = 0,
+    COLOR_MODE_GRADIENT = 1,
+    COLOR_MODE_FIRE = 2,
+    COLOR_MODE_DETAILED = 3,
 };
 
 /* Number of milliseconds between updates */
@@ -68,8 +78,6 @@ enum CPUGraphUpdateRate
     RATE_SLOWEST = 4,
 };
 
-enum { NUM_COLORS = 6 };
-
 enum CPUGraphColorNumber
 {
     BG_COLOR = 0,
@@ -77,13 +85,29 @@ enum CPUGraphColorNumber
     FG_COLOR2 = 2,
     FG_COLOR3 = 3,
     BARS_COLOR = 4,
-    SMT_ISSUES_COLOR = 5, /* NUM_COLORS-1 */
+    SMT_ISSUES_COLOR = 5,
+    FG_COLOR_SYSTEM = 6,
+    FG_COLOR_USER = 7,
+    FG_COLOR_NICE = 8,
+    FG_COLOR_IOWAIT = 9,
+
+    NUM_COLORS = 10,
 };
 
 struct CpuLoad
 {
     gint64 timestamp; /* Microseconds since 1970-01-01 UTC, or zero */
-    gfloat value;     /* Range: from 0.0 to 1.0 */
+
+    /* Range of float values: from 0.0 to 1.0 */
+
+    /* Overall CPU load: user + system + nice */
+    gfloat value;
+
+    /* Detailed CPU load */
+    gfloat system;
+    gfloat user;
+    gfloat nice;
+    gfloat iowait;
 } __attribute__((packed));
 
 struct CPUGraph
@@ -103,6 +127,7 @@ struct CPUGraph
     GtkWidget *tooltip_text;
 
     /* Settings */
+    XfconfChannel *channel;
     CPUGraphUpdateRate update_interval;
     guint size;
     CPUGraphMode mode;
@@ -120,6 +145,7 @@ struct CPUGraph
     bool has_bars:1;
     bool has_border:1;
     bool has_frame:1;
+    bool stats_smt:1;
     bool highlight_smt:1;
     bool non_linear:1;
     bool per_core:1;
@@ -137,9 +163,16 @@ struct CPUGraph
     std::vector<CpuData> cpu_data;  /* size == nr_cores+1 */
     Ptr0<Topology> topology;
     CpuStats stats;
+    std::vector<const CpuLoad *> nearest_cache;
+    std::vector<CpuLoad> non_linear_cache;
 
     ~CPUGraph();
 
+    bool
+    isSmtIssuesEnabled () const {
+        return stats_smt || (has_bars && highlight_smt);
+    }
+
     static void set_bars                 (const Ptr<CPUGraph> &base, bool bars);
     static void set_border               (const Ptr<CPUGraph> &base, bool border);
     static void set_color                (const Ptr<CPUGraph> &base, CPUGraphColorNumber number, const xfce4::RGBA &color);
@@ -153,10 +186,12 @@ struct CPUGraph
     static void set_per_core             (const Ptr<CPUGraph> &base, bool per_core);
     static void set_per_core_spacing     (const Ptr<CPUGraph> &base, guint spacing);
     static void set_size                 (const Ptr<CPUGraph> &base, guint width);
+    static void set_stats_smt            (const Ptr<CPUGraph> &base, bool stats_smt);
     static void set_smt                  (const Ptr<CPUGraph> &base, bool highlight_smt);
     static void set_startup_notification (const Ptr<CPUGraph> &base, bool startup_notification);
     static void set_tracked_core         (const Ptr<CPUGraph> &base, guint core);
     static void set_update_rate          (const Ptr<CPUGraph> &base, CPUGraphUpdateRate rate);
+    static void maybe_clear_smt_stats    (const Ptr<CPUGraph> &base);
 };
 
 guint get_update_interval_ms (CPUGraphUpdateRate rate);
