@@ -23,6 +23,10 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "cpu.h"
 #include "properties.h"
 #include "settings.h"
@@ -37,13 +41,16 @@ struct CPUGraphOptions
     const Ptr<CPUGraph> base;
 
     GtkColorButton  *color_buttons[NUM_COLORS] = {};
+    GtkWidget       *mode_combobox = NULL;
     GtkWidget       *color_mode_combobox = NULL;
+    GtkBox          *hbox_stats_smt = nullptr;
     GtkBox          *hbox_highlight_smt = NULL;
     GtkBox          *hbox_in_terminal = NULL;
     GtkBox          *hbox_per_core_spacing = NULL;
     GtkBox          *hbox_startup_notification = NULL;
     GtkToggleButton *per_core = NULL, *show_bars_checkbox = NULL;
     GtkLabel        *smt_stats = NULL;
+    GtkWidget       *notebook = nullptr;
     guint           timeout_id = 0;
 
     CPUGraphOptions(const Ptr<CPUGraph> &_base) : base(_base) {}
@@ -76,7 +83,8 @@ static GtkBox*    create_check_box (GtkBox *tab, GtkSizeGroup *sg, const gchar *
                                     GtkToggleButton **out_checkbox, const std::function<void (GtkToggleButton*)> &callback);
 static GtkWidget* create_drop_down (GtkBox *tab, GtkSizeGroup *sg, const gchar *name,
                                     const std::vector<std::string> &items, size_t init,
-                                    const std::function<void(GtkComboBox*)> &callback);
+                                    const std::function<void(GtkComboBox*)> &callback,
+                                    bool text_only = true);
 static void       setup_update_interval_option (GtkBox *vbox, GtkSizeGroup *sg, const Ptr<CPUGraphOptions> &data);
 static void       setup_tracked_core_option (GtkBox *vbox, GtkSizeGroup *sg, const Ptr<CPUGraphOptions> &data);
 static void       setup_size_option (GtkBox *vbox, GtkSizeGroup *sg, XfcePanelPlugin *plugin, const Ptr<CPUGraph> &base);
@@ -91,7 +99,7 @@ static GtkBox*    setup_per_core_spacing_option (GtkBox *vbox, GtkSizeGroup *sg,
 static void       change_color (GtkColorButton  *button, const Ptr<CPUGraph> &base, CPUGraphColorNumber number);
 static void       update_sensitivity (const Ptr<CPUGraphOptions> &data, bool initial = false);
 
-static xfce4::TimeoutResponse update_cb (const Ptr<CPUGraphOptions> &data);
+static void       update_stats_smt_cb (const Ptr<CPUGraphOptions> &data);
 
 void
 create_options (XfcePanelPlugin *plugin, const Ptr<CPUGraph> &base)
@@ -116,7 +124,7 @@ create_options (XfcePanelPlugin *plugin, const Ptr<CPUGraph> &base)
     xfce4::connect_response (GTK_DIALOG (dlg), [base, dlg](GtkDialog*, gint response) {
         gtk_widget_destroy (dlg);
         xfce_panel_plugin_unblock_menu (base->plugin);
-        write_settings (base->plugin, base);
+        Settings::write (base->plugin, base);
     });
 
     gtk_window_set_icon_name (GTK_WINDOW (dlg), "org.xfce.panel.cpugraph");
@@ -146,12 +154,36 @@ create_options (XfcePanelPlugin *plugin, const Ptr<CPUGraph> &base)
 
     const gchar *smt_issues_tooltip = _("Color used to highlight potentially suboptimal\nplacement of threads on CPUs with SMT");
 
+    auto startSmtStatsTimer = [dlg_data] {
+        gtk_widget_set_visible (gtk_notebook_get_nth_page (GTK_NOTEBOOK (dlg_data->notebook), 2), true);
+        update_stats_smt_cb (dlg_data);
+        dlg_data->timeout_id = xfce4::timeout_add (250, [dlg_data] {
+            update_stats_smt_cb (dlg_data);
+            return xfce4::TIMEOUT_AGAIN;
+        });
+    };
+    auto stopSmtStatsTimer = [dlg_data] {
+        gtk_widget_set_visible (gtk_notebook_get_nth_page (GTK_NOTEBOOK (dlg_data->notebook), 2), false);
+        dlg_data->removeTimer ();
+    };
+
     gtk_box_pack_start (vbox, gtk_separator_new (GTK_ORIENTATION_HORIZONTAL), FALSE, FALSE, BORDER/2);
+    dlg_data->hbox_stats_smt = create_check_box (vbox, sg, _("Display SMT statistics"),
+        base->stats_smt, NULL,
+        [dlg_data, startSmtStatsTimer, stopSmtStatsTimer](GtkToggleButton *button) {
+            CPUGraph::set_stats_smt (dlg_data->base, gtk_toggle_button_get_active (button));
+            if (dlg_data->base->stats_smt)
+                startSmtStatsTimer ();
+            else
+                stopSmtStatsTimer ();
+            dlg_data->base->maybe_clear_smt_stats (dlg_data->base);
+        });
     dlg_data->hbox_highlight_smt = create_check_box (vbox, sg, _("Highlight suboptimal SMT scheduling"),
         base->highlight_smt, NULL,
         [dlg_data](GtkToggleButton *button) {
             CPUGraph::set_smt (dlg_data->base, gtk_toggle_button_get_active (button));
             update_sensitivity (dlg_data);
+            dlg_data->base->maybe_clear_smt_stats (dlg_data->base);
         });
     setup_color_option (vbox, sg, dlg_data, SMT_ISSUES_COLOR, _("SMT issues color:"), smt_issues_tooltip, [base](GtkColorButton *button) {
         change_color (button, base, SMT_ISSUES_COLOR);
@@ -180,6 +212,18 @@ create_options (XfcePanelPlugin *plugin, const Ptr<CPUGraph> &base)
     setup_color_option (vbox2, sg, dlg_data, FG_COLOR3, _("Color 3:"), NULL, [base](GtkColorButton *button) {
         change_color (button, base, FG_COLOR3);
     });
+    setup_color_option (vbox2, sg, dlg_data, FG_COLOR_SYSTEM, _("System:"), NULL, [base](GtkColorButton *button) {
+        change_color (button, base, FG_COLOR_SYSTEM);
+    });
+    setup_color_option (vbox2, sg, dlg_data, FG_COLOR_USER, _("User:"), NULL, [base](GtkColorButton *button) {
+        change_color (button, base, FG_COLOR_USER);
+    });
+    setup_color_option (vbox2, sg, dlg_data, FG_COLOR_NICE, _("Nice:"), NULL, [base](GtkColorButton *button) {
+        change_color (button, base, FG_COLOR_NICE);
+    });
+    setup_color_option (vbox2, sg, dlg_data, FG_COLOR_IOWAIT, _("IO wait:"), NULL, [base](GtkColorButton *button) {
+        change_color (button, base, FG_COLOR_IOWAIT);
+    });
     setup_color_option (vbox2, sg, dlg_data, BG_COLOR, _("Background:"), NULL, [base](GtkColorButton *button) {
         change_color (button, base, BG_COLOR);
     });
@@ -211,19 +255,22 @@ create_options (XfcePanelPlugin *plugin, const Ptr<CPUGraph> &base)
     GtkBox *vbox3 = create_tab ();
     dlg_data->smt_stats = create_label_line (vbox3, "");
 
-    GtkWidget *notebook = gtk_notebook_new ();
-    gtk_container_set_border_width (GTK_CONTAINER (notebook), BORDER - 2);
-    gtk_notebook_append_page (GTK_NOTEBOOK (notebook), GTK_WIDGET (vbox2), gtk_label_new (_("Appearance")));
-    gtk_notebook_append_page (GTK_NOTEBOOK (notebook), GTK_WIDGET (vbox), gtk_label_new (_("Advanced")));
-    gtk_notebook_append_page (GTK_NOTEBOOK (notebook), GTK_WIDGET (vbox3), gtk_label_new (_("Stats")));
+    dlg_data->notebook = gtk_notebook_new ();
+    gtk_container_set_border_width (GTK_CONTAINER (dlg_data->notebook), BORDER - 2);
+    gtk_notebook_append_page (GTK_NOTEBOOK (dlg_data->notebook), GTK_WIDGET (vbox2), gtk_label_new (_("Appearance")));
+    gtk_notebook_append_page (GTK_NOTEBOOK (dlg_data->notebook), GTK_WIDGET (vbox), gtk_label_new (_("Advanced")));
+    gtk_notebook_append_page (GTK_NOTEBOOK (dlg_data->notebook), GTK_WIDGET (vbox3), gtk_label_new (_("Stats")));
 
     GtkWidget *content = gtk_dialog_get_content_area (GTK_DIALOG (dlg));
-    gtk_container_add (GTK_CONTAINER (content), notebook);
+    gtk_container_add (GTK_CONTAINER (content), dlg_data->notebook);
 
-    update_cb (dlg_data);
-    dlg_data->timeout_id = xfce4::timeout_add (100, [dlg_data]() { return update_cb(dlg_data); });
+    gtk_widget_show_all (dlg_data->notebook);
+
+    if (base->stats_smt)
+        startSmtStatsTimer ();
+    else
+        stopSmtStatsTimer ();
 
-    gtk_widget_show_all (notebook);
     update_sensitivity (dlg_data, true);
     gtk_widget_show (dlg);
 }
@@ -297,13 +344,43 @@ create_check_box (GtkBox *tab, GtkSizeGroup *sg, const gchar *name, bool init,
 static GtkWidget*
 create_drop_down (GtkBox *tab, GtkSizeGroup *sg, const gchar *name,
                   const std::vector<std::string> &items, size_t init,
-                  const std::function<void(GtkComboBox*)> &callback)
+                  const std::function<void(GtkComboBox*)> &callback,
+                  bool text_only)
 {
     GtkBox *hbox = create_option_line (tab, sg, name, NULL);
 
-    GtkWidget *combo = gtk_combo_box_text_new ();
-    for (const std::string &item : items)
-        gtk_combo_box_text_append (GTK_COMBO_BOX_TEXT (combo), NULL, item.c_str());
+    GtkWidget *combo;
+    if (text_only)
+    {
+        combo = gtk_combo_box_text_new ();
+        for (const std::string &item : items)
+            gtk_combo_box_text_append (GTK_COMBO_BOX_TEXT (combo), NULL, item.c_str());
+    }
+    else
+    {
+        GtkListStore *list_store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_BOOLEAN);
+
+        GtkTreeIter iter;
+        for (const std::string &item : items)
+        {
+            gtk_list_store_append (list_store, &iter);
+            gtk_list_store_set (list_store, &iter,
+                                0, item.c_str(),
+                                1,  true,
+                                -1);
+        }
+
+        combo = gtk_combo_box_new_with_model (GTK_TREE_MODEL (list_store));
+
+        g_object_unref (list_store);
+
+        GtkCellRenderer *cell = gtk_cell_renderer_text_new ();
+        gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo), cell, true);
+        gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo), cell,
+                                        "text", 0,
+                                        "sensitive", 1,
+                                        nullptr);
+    }
     gtk_combo_box_set_active (GTK_COMBO_BOX (combo), init);
     gtk_box_pack_start (GTK_BOX (hbox), combo, FALSE, FALSE, 0);
 
@@ -450,11 +527,9 @@ setup_mode_option (GtkBox *vbox, GtkSizeGroup *sg, const Ptr<CPUGraphOptions> &d
         case MODE_GRID:       selected = 4; break;
     }
 
-    create_drop_down (vbox, sg, _("Mode:"), items, selected,
+    data->mode_combobox = create_drop_down (vbox, sg, _("Mode:"), items, selected,
         [data](GtkComboBox *combo) {
-            /* 'Disabled' mode was introduced in 1.1.0 as '-1'
-             * for this reason we need to decrement the selected value */
-            gint active = gtk_combo_box_get_active (combo) - 1;
+            gint active = gtk_combo_box_get_active (combo);
             CPUGraphMode mode;
 
             switch (active)
@@ -475,7 +550,7 @@ setup_mode_option (GtkBox *vbox, GtkSizeGroup *sg, const Ptr<CPUGraphOptions> &d
                 gtk_toggle_button_set_active (data->show_bars_checkbox, TRUE);
 
             update_sensitivity (data);
-        });
+        }, false);
 }
 
 static void
@@ -485,6 +560,9 @@ setup_color_mode_option (GtkBox *vbox, GtkSizeGroup *sg, const Ptr<CPUGraphOptio
         _("Solid"),
         _("Gradient"),
         _("Fire"),
+#if defined (__linux__) || defined (__FreeBSD_kernel__)
+        _("Detailed"),
+#endif
     };
 
     data->color_mode_combobox = create_drop_down (
@@ -492,7 +570,7 @@ setup_color_mode_option (GtkBox *vbox, GtkSizeGroup *sg, const Ptr<CPUGraphOptio
         [data](GtkComboBox *combo) {
             CPUGraph::set_color_mode (data->base, gtk_combo_box_get_active (combo));
             update_sensitivity (data);
-        });
+        }, false);
 }
 
 static void
@@ -525,22 +603,72 @@ update_sensitivity (const Ptr<CPUGraphOptions> &data, bool initial)
     gtk_widget_set_sensitive (GTK_WIDGET (data->per_core), per_core);
     gtk_widget_set_sensitive (GTK_WIDGET (data->hbox_per_core_spacing), per_core && base->per_core);
 
-    gtk_widget_set_sensitive (gtk_widget_get_parent (GTK_WIDGET (data->color_buttons[FG_COLOR2])),
-                              base->color_mode != 0 || base->mode == MODE_LED || base->mode == MODE_GRID);
-    gtk_widget_set_sensitive (gtk_widget_get_parent (GTK_WIDGET (data->color_buttons[FG_COLOR3])),
-                              base->color_mode != 0 && base->mode == MODE_LED);
-
-    gtk_widget_set_sensitive (gtk_widget_get_parent (GTK_WIDGET (data->color_buttons[BARS_COLOR])), base->has_bars);
-    gtk_widget_set_sensitive (gtk_widget_get_parent (GTK_WIDGET (data->color_buttons[SMT_ISSUES_COLOR])),
+    auto get_color_button_parent = [&](CPUGraphColorNumber color) {
+        return gtk_widget_get_parent (GTK_WIDGET (data->color_buttons[color]));
+    };
+    auto set_colors_visibility = [&](bool detailed_mode) {
+        gtk_widget_set_visible (get_color_button_parent (FG_COLOR1), !detailed_mode);
+        gtk_widget_set_visible (get_color_button_parent (FG_COLOR2), !detailed_mode);
+        gtk_widget_set_visible (get_color_button_parent (FG_COLOR3), !detailed_mode);
+        gtk_widget_set_visible (get_color_button_parent (FG_COLOR_SYSTEM), detailed_mode);
+        gtk_widget_set_visible (get_color_button_parent (FG_COLOR_USER), detailed_mode);
+        gtk_widget_set_visible (get_color_button_parent (FG_COLOR_NICE), detailed_mode);
+        gtk_widget_set_visible (get_color_button_parent (FG_COLOR_IOWAIT), detailed_mode);
+    };
+    if (base->color_mode == COLOR_MODE_DETAILED)
+    {
+        set_colors_visibility (true);
+        gtk_widget_set_sensitive (get_color_button_parent(FG_COLOR_SYSTEM),
+                                  base->mode != MODE_DISABLED);
+        gtk_widget_set_sensitive (get_color_button_parent(FG_COLOR_USER),
+                                  base->mode != MODE_DISABLED);
+        gtk_widget_set_sensitive (get_color_button_parent(FG_COLOR_NICE),
+                                  base->mode != MODE_DISABLED);
+        gtk_widget_set_sensitive (get_color_button_parent(FG_COLOR_IOWAIT),
+                                  base->mode != MODE_DISABLED);
+    }
+    else
+    {
+        set_colors_visibility (false);
+        gtk_widget_set_sensitive (get_color_button_parent(FG_COLOR2),
+                                  base->mode != MODE_DISABLED && (base->color_mode != COLOR_MODE_SOLID || base->mode == MODE_LED || base->mode == MODE_GRID));
+        gtk_widget_set_sensitive (get_color_button_parent(FG_COLOR3),
+                                  base->mode != MODE_DISABLED && (base->color_mode != COLOR_MODE_SOLID && base->mode == MODE_LED));
+    }
+    gtk_widget_set_sensitive (get_color_button_parent(FG_COLOR1),
+                              base->mode != MODE_DISABLED);
+    gtk_widget_set_sensitive (get_color_button_parent(BARS_COLOR), base->has_bars);
+    gtk_widget_set_sensitive (get_color_button_parent(SMT_ISSUES_COLOR),
                               base->has_bars && base->highlight_smt && base->topology && base->topology->smt);
 
     gtk_widget_set_sensitive (gtk_widget_get_parent (data->color_mode_combobox),
                               base->mode != MODE_DISABLED && base->mode != MODE_GRID);
     gtk_widget_set_sensitive (GTK_WIDGET (data->show_bars_checkbox), base->mode != MODE_DISABLED);
+
+    /* Set combobox items (in)sensitive depending on chosen modes. Detailed color mode is supported for:
+     * - MODE_NORMAL
+     * - MODE_NO_HISTORY
+     */
+    if (GtkTreeModel *model = gtk_combo_box_get_model (GTK_COMBO_BOX (data->mode_combobox)))
+    {
+        const bool is_detailed = (data->base->color_mode == COLOR_MODE_DETAILED);
+        GtkTreeIter iter;
+        if (gtk_tree_model_iter_nth_child (model, &iter, nullptr, MODE_LED))
+            gtk_list_store_set (GTK_LIST_STORE (model), &iter, 1, !is_detailed, -1);
+        if (gtk_tree_model_iter_nth_child (model, &iter, nullptr, MODE_GRID))
+            gtk_list_store_set (GTK_LIST_STORE (model), &iter, 1, !is_detailed, -1);
+    }
+    if (GtkTreeModel *model = gtk_combo_box_get_model (GTK_COMBO_BOX (data->color_mode_combobox)))
+    {
+        const bool is_led = (data->base->mode == MODE_LED);
+        GtkTreeIter iter;
+        if (gtk_tree_model_iter_nth_child (model, &iter, nullptr, COLOR_MODE_DETAILED))
+            gtk_list_store_set (GTK_LIST_STORE (model), &iter, 1, !is_led, -1);
+    }
 }
 
-static xfce4::TimeoutResponse
-update_cb (const Ptr<CPUGraphOptions> &data)
+static void
+update_stats_smt_cb (const Ptr<CPUGraphOptions> &data)
 {
     const Ptr<CPUGraph> base = data->base;
     std::string smt_text;
@@ -600,6 +728,4 @@ update_cb (const Ptr<CPUGraphOptions> &data)
         gtk_label_set_text (data->smt_stats, smt_text.c_str());
         gtk_widget_set_tooltip_text (GTK_WIDGET (data->smt_stats), show_tooltip ? data->smt_stats_tooltip().c_str() : "");
     }
-
-    return xfce4::TIMEOUT_AGAIN;
 }
